@@ -3,7 +3,7 @@
 #----------------------------------------------------------------------
 # xotl.ql.core
 #----------------------------------------------------------------------
-# Copyright (c) 2012 Merchise Autrement and Contributors
+# Copyright (c) 2012, 2013 Merchise Autrement and Contributors
 # All rights reserved.
 #
 # This is free software; you can redistribute it and/or modify it under
@@ -37,7 +37,8 @@ from xoutil.types import Unset
 from xoutil.objects import validate_attrs
 from xoutil.context import context
 from xoutil.proxy import UNPROXIFING_CONTEXT
-from xoutil.decorators import decorator
+from xoutil.decorator.meta import decorator
+from xoutil.decorator.compat import metaclass
 from xoutil.aop.basic import complementor
 
 from zope.component import getUtility
@@ -45,11 +46,10 @@ from zope.interface import implementer
 from zope.interface import alsoProvides, noLongerProvides
 
 from xotl.ql.expressions import _true, _false, ExpressionTree, OperatorType
-from xotl.ql.expressions import UNARY, BINARY, N_ARITY
+from xotl.ql.expressions import UNARY, BINARY, N_ARITY, EXPRESSION_CONTEXT
 from xotl.ql.interfaces import (ITerm,
                                 IBoundTerm,
                                 IGeneratorToken,
-                                IQueryPart,
                                 IExpressionTree,
                                 IExpressionCapable,
                                 IQueryTranslator,
@@ -62,60 +62,80 @@ __docstring_format__ = 'rst'
 __author__ = 'manu'
 
 
-__all__ = (b'this', b'these',)
+__all__ = (str('this'), str('these'),)
 
 
 # A thread-local namespace to avoid using context. Just to test if this
 # avoid the context's bug.
-_local = threading.local()
+# _local = threading.local()
 
 
-def _get_bubbles_stack():
-    unset = object()
-    stack = getattr(_local, 'bubbles', unset)
-    if stack is unset:
-        stack = _local.bubbles = []
-    return stack
+# def _get_bubbles_stack():
+#     unset = object()
+#     stack = getattr(_local, 'bubbles', unset)
+#     if stack is unset:
+#         stack = _local.bubbles = []
+#     return stack
 
 
-def _create_and_push_bubble():
-    'Creates a bubble and pushes it to the local stack'
-    bubbles = _get_bubbles_stack()
-    bubble = QueryParticlesBubble()
-    bubbles.append(bubble)
-    return bubble
+# def _create_and_push_bubble():
+#     'Creates a bubble and pushes it to the local stack'
+#     bubbles = _get_bubbles_stack()
+#     bubble = QueryParticlesBubble()
+#     bubbles.append(bubble)
+#     return bubble
 
 
-def _pop_bubble():
-    'Removes the top-most bubble from the bubble stack'
-    bubbles = _get_bubbles_stack()
-    return bubbles.pop(-1)
+# def _pop_bubble():
+#     'Removes the top-most bubble from the bubble stack'
+#     bubbles = _get_bubbles_stack()
+#     return bubbles.pop(-1)
 
 
-def _get_current_bubble():
-    'Returns the top-most bubble'
-    bubbles = _get_bubbles_stack()
-    return bubbles[-1]
+# def _get_current_bubble():
+#     'Returns the top-most bubble'
+#     bubbles = _get_bubbles_stack()
+#     return bubbles[-1]
 
 
-def _emit_part(part):
+def _emit_part(part, quiet=False):
     'Emits a particle to the current bubble'
-    bubble = _get_current_bubble()
-    bubble.capture_part(part)
+    # bubble = _get_current_bubble()
+    # bubble.capture_part(part)
+    bubble = context[EXPRESSION_CONTEXT].data.get('bubble', None)
+    if bubble:
+        bubble.capture_part(part)
+    elif not quiet:
+        raise RuntimeError('There should be context')
 
 
-def _emit_token(token):
+def _emit_token(token, quiet=False):
     'Emits a token to the current bubble'
-    bubble = _get_current_bubble()
-    bubble.capture_token(token)
+    # bubble = _get_current_bubble()
+    # bubble.capture_token(token)
+    bubble = context[EXPRESSION_CONTEXT].data.get('bubble', None)
+    if bubble:
+        bubble.capture_token(token)
+    elif not quiet:
+        raise RuntimeError('There should be context')
+
 
 
 @implementer(ITerm)
 class Term(object):
-    '''
-    The type of the :obj:`this` symbol: an unnamed object that may placed in
+    '''The type of the :obj:`this` symbol: an unnamed object that may placed in
     queries and whose interpretation depends on the query context and the
     context in which `this` symbol is used inside the query itself.
+
+    :param name: The name of the term. It sould be a valid Python identifier.
+
+
+    :param parent: Another :class:`ITerm` instance this term is "drawn"
+                   from.
+
+    :param binding: The :class:`IGeneratorToken` this term is bound to in a
+                    query context.
+
     '''
 
     _counter = count(1)
@@ -196,6 +216,28 @@ class Term(object):
     def _newname(cls):
         return '::i{count}'.format(count=next(cls._counter))
 
+    def clone(self, **kwargs):
+        '''Clones the current term with possible variations.
+
+        If keyword arguments are passed those will be used to modify the cloned
+        term.
+
+        '''
+        try:
+            self.validate_name(self.name)
+            should_invalidate = False
+        except:
+            should_invalidate = True
+        attrs = dict(name=self.name, parent=self.parent,
+                     binding=self.binding)
+        attrs.update(kwargs)
+        if should_invalidate:
+            with context('_INVALID_THESE_NAME'):
+                result = type(self)(**attrs)
+        else:
+            result = type(self)(**attrs)
+        return result
+
     def __getattribute__(self, attr):
         # Notice we can't use the __getattr__ way because then things like
         # ``this.name`` would not work properly.
@@ -204,7 +246,9 @@ class Term(object):
             context[UNPROXIFING_CONTEXT]):
             return get(attr)
         else:
-            return Term(name=attr, parent=self)
+            result = Term(name=attr, parent=self)
+            _emit_part(result, True)
+            return result
 
     def __call__(self, *args, **kwargs):
         with context(UNPROXIFING_CONTEXT):
@@ -216,44 +260,58 @@ class Term(object):
             raise TypeError()
 
     def __iter__(self):
-        '''
-        Yields a single instance of :class:`query part
-        <xotl.ql.interfaces.IQueryPart>` that wraps `self`.
+        '''Yields a single instance of a :class:`bound term
+        <xotl.ql.interfaces.IBoundTerm>` that is a clone of the current.
 
         This allows an idiomatic way to express queries::
 
-            >>> parent, child = next((parent, child)
-            ...                            for parent in this('parent')
-            ...                            for child in parent.children)
-            >>> (parent, child)    # doctest: +ELLIPSIS
-            (<...this('parent')...>, <...this('parent').children...>)
+            query_expression = ((parent, child)
+                                for parent in this('parent')
+                                for child in parent.children)
 
-        .. warning::
+        See :class:`these`.
 
-           We have used `next` here directly over the comprehensions, but the
-           query language **does not** support this kind of construction.
-
-           Queries must be built by calling the :func:`these` passing the
-           comprehension as its first argument.
         '''
         with context(UNPROXIFING_CONTEXT):
             name = self.name
             parent = self.parent
         with context(UNPROXIFING_CONTEXT):
-            if name:
-                token = GeneratorToken(expression=self)
-                bound_term = Term(name, parent=parent, binding=token)
-            else:
+            if not name:
                 # When iterating an instance without a name (i.e the `this`
                 # object), we should generate a new name (of those simple
                 # mortals can't use)
                 with context('_INVALID_THESE_NAME'):
                     name = self._newname()
-                    term = Term(name, parent=parent)
-                    token = GeneratorToken(expression=term)
-                    bound_term = Term(name, parent=parent, binding=token)
-        instance = QueryPart(expression=bound_term)
-        _emit_token(token)
+                    bound_term = Term(name, parent=parent, binding=None)
+            else:
+                bound_term = Term(name, parent=parent, binding=None)
+            token = GeneratorToken(expression=bound_term)
+            bound_term.binding = token
+            # XXX: Since bound_terms might span accross subqueries, there might
+            # be several stacked bubbles around. For instance::
+            #
+            #    (parent
+            #        for parent in this('parent')
+            #        if any_(child for child in parent.children if child.age < 6))
+            #
+            # In this case, the enclosed parent.children emits to top-most
+            # bubble, but when the iter is invoked by `any_` to resolve its
+            # argument a second bubble is in place and the hack (XXX) below
+            # won't work. Furthermore, the token's expression is the bound-term
+            # while the emmited part was the original one. So we keep the
+            # original_term so that operations that resolver their arguments as
+            # subquery may remove the escaped part.
+            bound_term.original_term = self
+        instance = bound_term
+        # XXX: When emiting a token in the context of query, if this token's
+        # expression is also a part, then it was emitted without binding, so we
+        # need to manually check this case here
+        bubble = context[EXPRESSION_CONTEXT].data.get('bubble', None)
+        if bubble:
+            parts = bubble._parts
+            if parts and self is parts[-1]:
+                parts.pop(-1)
+            _emit_token(token)
         yield instance
 
     def __str__(self):
@@ -272,6 +330,9 @@ class Term(object):
     def __repr__(self):
         return '<%s at 0x%x>' % (str(self), id(self))
 
+    def __hash__(self):
+        return hash(str(self))
+
     def __eq__(self, other):
         '''
             >>> with context(UNPROXIFING_CONTEXT):
@@ -658,6 +719,8 @@ class QueryParticlesBubble(object):
                 return result
         elif ITerm.providedBy(expression):
             return expression.parent is top
+        elif IQueryObject.providedBy(expression):
+            return False
         else:
             raise TypeError('Parts should be either these instance or '
                             'expression trees; not %s' % type(expression))
@@ -696,21 +759,21 @@ class QueryParticlesBubble(object):
            lost.
 
         :param part: The emitted query part
-        :type part: :class:`IQueryPart`
+        :type part: :class:`IExpressionCapable`
         '''
         with context(UNPROXIFING_CONTEXT):
-            if provides_all(part, IQueryPart):
-                expression = part.expression
-                parts = self._parts
-                if parts:
-                    mergable = self.mergable
-                    while parts and mergable(expression):
-                        top = parts.pop()
-                        self._particles.remove(top)
-                parts.append(expression)
-                self._particles.append(expression)
+            if provides_all(part, IExpressionCapable):
+                expression = part
             else:
                 assert False
+            parts = self._parts
+            if parts:
+                mergable = self.mergable
+                while parts and mergable(expression):
+                    top = parts.pop()
+                    self._particles.remove(top)
+            parts.append(expression)
+            self._particles.append(expression)
 
     def capture_token(self, token):
         '''Captures an emitted token.
@@ -739,20 +802,21 @@ class QueryParticlesBubble(object):
                 top = parts.pop(-1)
                 if token.expression is not top:
                     parts.append(top)
-        if token not in tokens:
-            tokens.append(token)
-            self._particles.append(token)
+        tokens.append(token)
+        self._particles.append(token)
 
 
 class _QueryObjectType(type):
-    def these(self, comprehension, **kwargs):
+    def build_from_generator(self, comprehension, **kwargs):
         '''Builds a :term:`query object` from a :term:`query expression` given
-        by a comprehension.
+        by a generator object.
 
         :param comprehension: The :term:`query expression` to be processed.
+        :type comprehension: GeneratorType
 
-        :param ordering: The ordering expressions.
-        :type ordering: A tuple of ordering expressions.
+        :param ordering: A function that receives as many arguments as there
+             are selections in the query and returns a tuple of :ref:`ordering
+             expressions <ordering-expressions>`.
 
         :param partition: A slice `(offset, limit, step)` that represents the
                           part of the result set to be retrieved.
@@ -768,18 +832,12 @@ class _QueryObjectType(type):
         :param offset: Individually express the offset of the `partition`
                        parameter.
 
-        :type offset: int or None
-
         :param limit: Individually express the limit of the `partition`
                       parameter.
 
-        :type limit: int or None
-
         :param step: Individually express the step of the `partition`
                      parameter.
 
-        :type step: int or None
-
         :returns: An :class:`~xotl.ql.interfaces.IQueryObject` instance that
                   represents the QueryObject expressed by the `comprehension`
                   and the `kwargs`.
@@ -790,35 +848,47 @@ class _QueryObjectType(type):
         .. note::
 
            All others keyword arguments are copied to the
-           :attr:`~xotl.ql.interface.IQueryObject.params` attribute, so that
+           :attr:`~xotl.ql.interfaces.IQueryObject.params` attribute, so that
            :term:`query translators <query translator>` may use them.
 
         '''
         from types import GeneratorType
         assert isinstance(comprehension, GeneratorType)
-        bubble = _create_and_push_bubble()
-        try:
+        with context(EXPRESSION_CONTEXT) as _context:
+            bubble = _context.data.bubble = QueryParticlesBubble()
             selected_parts = next(comprehension)
-        finally:
-            b = _pop_bubble()
-            assert b is bubble
+        selected_parts_type = type(selected_parts)
         with context(UNPROXIFING_CONTEXT):
             if not isinstance(selected_parts, (list, tuple)):
                 selected_parts = (selected_parts,)
-            selected_parts = tuple(reversed(selected_parts))
             selection = []
-            tokens = bubble.tokens
             filters = bubble.parts
-            for part in selected_parts:
-                expr = part.expression
-                if filters and expr is filters[-1]:
+            # Selections that are parts' were emitted last and are disguised as
+            # filters at the top of the bubble.parts, this should be fixed:
+            for part in reversed(selected_parts):
+                if filters and part is filters[-1]:
                     filters.pop(-1)
-                selection.append(expr)
+                selection.append(part)
             query = self()
-            query.selection = tuple(reversed(selection))
-            query.tokens = tuple(set(token.expression for token in tokens))
-            query.filters = tuple(set(filters))
-            query.ordering = kwargs.get('ordering', None)
+            if issubclass(selected_parts_type, (list, tuple)):
+                query.selection = selected_parts_type(reversed(selection))
+            else:
+                assert len(selection) == 1
+                query.selection = selection[0]
+            tokens = bubble.tokens
+            query.tokens = tuple(tokens)
+            query.filters = tuple(filters)
+        # Step out ot the context for a moment in order to create the ordering
+        # expressions (otherwise things like term.attr would fail).
+        orderby = kwargs.get('ordering', None)
+        if orderby:
+            # from xoutil.proxy import unboxed
+            ordering = orderby(*selection)
+        else:
+            ordering = None
+        # Now continue inside a comfortable execution context.
+        with context(UNPROXIFING_CONTEXT):
+            query.ordering = ordering
             partition = kwargs.get('partition', None)
             offset = kwargs.get('offset', None)
             limit = kwargs.get('limit', None)
@@ -841,7 +911,7 @@ class _QueryObjectType(type):
             first_arg, args = args[0], args[1:]
             if not args:
                 if isinstance(first_arg, GeneratorType):
-                    return self.these(first_arg, **kwargs)
+                    return self.build_from_generator(first_arg, **kwargs)
                 # TODO: Other types of queries
 
         result = super(self, self).__new__(self)
@@ -850,12 +920,11 @@ class _QueryObjectType(type):
 
 
 @implementer(IQueryObject)
+@metaclass(_QueryObjectType)
 class QueryObject(object):
     '''
     Represents a query. See :class:`xotl.ql.interfaces.IQueryObject`.
     '''
-    __metaclass__ = _QueryObjectType
-
     def __init__(self):
         self._selection = None
         self.tokens = None
@@ -872,8 +941,8 @@ class QueryObject(object):
     def selection(self, value):
         ok = lambda v: isinstance(v, (ExpressionTree, Term))
         if ok(value):
-            self._selection = (value,)
-        elif isinstance(value, tuple) and all(ok(v) for v in value):
+            self._selection = value
+        elif isinstance(value, (tuple, list)) and all(ok(v) for v in value):
             self._selection = value
         # TODO: Include dict
         else:
@@ -943,7 +1012,7 @@ class QueryObject(object):
             conf = getUtility(IQueryConfiguration)
             name = getattr(conf, 'default_translator_name', None)
             translator = getUtility(IQueryTranslator,
-                                    name if name else b'default')
+                                    name if name else str('default'))
             query_plan = translator.build_plan(self)
             state = self._query_state = query_plan()
         result = next(state, (Unset, Unset))
@@ -963,23 +1032,15 @@ class QueryObject(object):
         'Creates a subquery'
         raise NotImplementedError
 
-
 these = QueryObject
 
 
 @implementer(IGeneratorToken)
 class GeneratorToken(object):
-    '''
-    Represents a token in the syntactical tree that is used as generator.
-
-    This object is also an :class:`~xotl.ql.interfaces.IQueryPartContainer`,
-    because in this implementation we need to record each time an
-    :class:`~xotl.ql.interfaces.IQueryPart` is created in order to later
-    retrieve the filters related to this generator token.
+    '''Represents a token in the syntactical tree that is used as
+    generator. See :class:`xotl.ql.interfaces.IGeneratorToken`.
 
     '''
-    __slots__ = ('_expression', '_parts')
-
     # TODO: Representation of grouping with dicts.
     def __init__(self, expression):
         assert provides_any(expression, ITerm)
@@ -991,6 +1052,11 @@ class GeneratorToken(object):
             if isinstance(other, GeneratorToken):
                 return self._expression == other._expression
 
+    def __hash__(self):
+        with context(UNPROXIFING_CONTEXT):
+            signature = '<tk: %s>' % str(self._expression)
+        return hash(signature)
+
     def __repr__(self):
         return '<tk: %r>' % self._expression
 
@@ -998,145 +1064,14 @@ class GeneratorToken(object):
     def expression(self):
         return self._expression
 
-
-def _query_part_method(target):
-    '''Decorator of every method in QueryPart that emits its result to
-    the "active" particle bubble.'''
-    def inner(self, *args, **kwargs):
-        result = target(self, *args, **kwargs)
-        _emit_part(result)
-        return result
-    inner.__name__ = target.__name__
-    return inner
-
-
-def _build_unary_operator(operation):
-    method_name = operation._method_name
-    def method(self):
-        result = QueryPart(expression=operation(self))
-        return result
-    method.__name__ = method_name
-    return _query_part_method(method)
-
-
-def _build_binary_operator(operation, inverse=False):
-    if not inverse:
-        method_name = operation._method_name
-    else:
-        method_name = operation._rmethod_name
-    if method_name:
-        def method(self, *others):
-            if not inverse:
-                result = QueryPart(expression=operation(self, *others))
-            else:
-                assert operation._arity == BINARY
-                other = others[0]
-                result = QueryPart(expression=operation(other, self))
-            return result
-        method.__name__ = method_name
-        return _query_part_method(method)
-
-
-_part_operations = {operation._method_name:
-                    _build_unary_operator(operation)
-                 for operation in OperatorType.operators
-                    if getattr(operation, 'arity', None) is UNARY}
-_part_operations.update({operation._method_name:
-                        _build_binary_operator(operation)
-                      for operation in OperatorType.operators
-                        if getattr(operation, 'arity', None) in (BINARY, N_ARITY)})
-
-_part_operations.update({operation._rmethod_name:
-                        _build_binary_operator(operation, True)
-                      for operation in OperatorType.operators
-                        if getattr(operation, 'arity', None) is BINARY and
-                           getattr(operation, '_rmethod_name', None)})
-
-
-QueryPartOperations = type(b'QueryPartOperations', (object,), _part_operations)
-
-
-class _QueryPartType(type):
-    def _target_(self, part):
-        from xoutil.proxy import unboxed
-        return unboxed(part).expression
-
-
-@implementer(IQueryPart, ITerm)
-@complementor(QueryPartOperations)
-class QueryPart(object):
-    '''A class that wraps either :class:`Term` or :class:`ExpressionTree` that
-    implements the :class:`xotl.ql.interfaces.IQueryPart` interface.
-
-    To build a query object from a comprehension like in::
-
-        these(count(parent.children) for parent in this if parent.age > 34)
-
-    We need to differentiate the IF (``parent.age > 34``) part of the
-    comprehension from the SELECTION (``count(parent.children)``); which in the
-    general case are both expressions.
-
-    '''
-    __metaclass__ = _QueryPartType
-    __slots__ = ('_expression')
-
-    def __init__(self, **kwargs):
-        with context(UNPROXIFING_CONTEXT):
-            self._expression = expression = kwargs.get('expression')
-            assert IExpressionCapable.providedBy(expression)
-
-    @property
-    def expression(self):
-        return self._expression
-
     @expression.setter
     def expression(self, value):
-        if provides_any(value, IExpressionCapable):
-            self._expression = value
-        else:
-            raise TypeError('QueryParts wraps IExpressionCapable objects only')
+        self._expression = value
 
-    def __iter__(self):
-        with context(UNPROXIFING_CONTEXT):
-            expression = self.expression
-            # XXX: In cases of sub-queries the part will be emitted, but the
-            #      iter will be hold, so the token won't be emitted and the
-            #      part won't be removed. So we're bringing this check back.
-            bubble = _get_current_bubble()
-            assert bubble
-            if bubble._parts and expression is bubble._parts[-1]:
-                bubble._parts.pop(-1)
-            return iter(expression)
-
-    def __str__(self):
-        with context(UNPROXIFING_CONTEXT):
-            instance = self.expression
-            result = str(instance)
-        return '<qp: %s>' % result
-    __repr__ = __str__
-
-    def __getattribute__(self, attr):
-        get = super(QueryPart, self).__getattribute__
-        if context[UNPROXIFING_CONTEXT]:
-            return get(attr)
-        else:
-            with context(UNPROXIFING_CONTEXT):
-                instance = get('expression')
-            result = QueryPart(expression=getattr(instance, attr))
-            _emit_part(result)
-            return result
-
-    @_query_part_method
-    def __call__(self, *args, **kwargs):
-        with context(UNPROXIFING_CONTEXT):
-            instance = self.expression
-        result = QueryPart(expression=instance(*args, **kwargs))
-        return result
 
 @decorator
 def thesefy(target, name=None):
-    '''
-    Takes in a class and injects it an `__iter__` so that the class may take
+    '''Takes in a class and injects it an `__iter__` so that the class may take
     part of `this` in :term:`query expressions <query expression>`.
 
         >>> @thesefy
@@ -1154,12 +1089,13 @@ def thesefy(target, name=None):
         ...                if which.name.startswith('A'))
 
     This is only useful if your real class does not have a metaclass of its own
-    that do that. However, if you do have a metaclass with an `__iter__` method
-    it should either return an `IQueryPart` instance or a `generator object`.
+    that does that already. However, if you do have a metaclass with an
+    `__iter__` method it must return a `generator object`: returning anything
+    else and "thesefying" is a TypeError.
 
-    Optionally (usually for debugging purposes only) you may pass a name to
-    the decorator that will be used as the name for the internally generated
-    :class:`Term` instance.
+    Optionally (useful for debugging purposes only) you may pass a name to the
+    decorator that will be used as the name for the internally generated
+    :class:`Term` instance::
 
         >>> @thesefy('Entity')
         ... class Entity(object):
@@ -1169,24 +1105,16 @@ def thesefy(target, name=None):
         >>> q.selection        # doctest: +ELLIPSIS
         (<this('Entity') at 0x...>,)
 
-    This way it's easier to create tests::
-
-        >>> filters = q.filters
-        >>> expected_is = is_instance(this('Entity'), Entity)
-        >>> expected_filter = this('Entity').name.startswith('A')
-
-        >>> any(unboxed(expected_is) == f for f in filters)
-        True
-
-        >>> any(unboxed(expected_filter) == f for f in filters)
-        True
+    This way it's easier to create tests.
 
     '''
-    from xoutil.objects import nameof
     class new_meta(type(target)):
         def __new__(cls, name, bases, attrs):
-            return super(new_meta, cls).__new__(cls, nameof(target),
-                                                bases, attrs)
+            from xoutil.iterators import dict_update_new
+            baseattrs = {'__doc__': getattr(bases[0], '__doc__', ''),
+                         '__module__': getattr(bases[0], '__module__', '')}
+            dict_update_new(attrs, baseattrs)
+            return super(new_meta, cls).__new__(cls, name, bases, attrs)
         def __iter__(self):
             from types import GeneratorType
             try:
@@ -1196,8 +1124,6 @@ def thesefy(target, name=None):
             if isinstance(result, GeneratorType):
                 for item in result:
                     yield item
-            elif result is not Unset and IQueryPart.providedBy(result):
-                    yield result
             elif result is Unset:
                 from xotl.ql.expressions import is_instance
                 query_part = next(iter(this(name)))
@@ -1208,7 +1134,5 @@ def thesefy(target, name=None):
                                 '__iter__ that does not support thesefy'
                                 .format(target=target))
 
-    class new_class(target):
-        __metaclass__ = new_meta
-    new_class.__doc__ = getattr(target, '__doc__', None)
+    new_class = new_meta(target.__name__, (target, ), {})
     return new_class
